package sun.util.locale.provider;

import java.util.spi.TimeZoneNameProvider;
import java.util.spi.LocaleNameProvider;
import java.util.spi.CurrencyNameProvider;
import java.util.spi.CalendarNameProvider;
import java.util.spi.CalendarDataProvider;
import java.text.NumberFormat;
import java.text.spi.NumberFormatProvider;
import java.text.DecimalFormatSymbols;
import java.text.spi.DecimalFormatSymbolsProvider;
import java.text.DateFormatSymbols;
import java.text.spi.DateFormatSymbolsProvider;
import java.text.DateFormat;
import java.text.spi.DateFormatProvider;
import java.text.Collator;
import java.text.spi.CollatorProvider;
import java.text.BreakIterator;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;
import java.text.spi.BreakIteratorProvider;
import java.util.Locale;
import java.util.Map;
import java.security.PrivilegedActionException;
import java.security.AccessController;
import java.util.Iterator;
import java.util.ServiceLoader;
import java.security.PrivilegedExceptionAction;
import java.util.spi.LocaleServiceProvider;

public class SPILocaleProviderAdapter extends AuxLocaleProviderAdapter
{
    public SPILocaleProviderAdapter() {
    }
    
    @Override
    public LocaleProviderAdapter.Type getAdapterType() {
        return LocaleProviderAdapter.Type.SPI;
    }
    
    @Override
    protected <P extends LocaleServiceProvider> P findInstalledProvider(final Class<P> clazz) {
        try {
            return (P)AccessController.doPrivileged(new PrivilegedExceptionAction<P>() {
                @Override
                public P run() {
                    LocaleServiceProvider localeServiceProvider = null;
                    for (LocaleServiceProvider localeServiceProvider2 : ServiceLoader.loadInstalled(clazz)) {
                        if (localeServiceProvider == null) {
                            try {
                                localeServiceProvider = (LocaleServiceProvider)Class.forName(SPILocaleProviderAdapter.class.getCanonicalName() + "$" + clazz.getSimpleName() + "Delegate").newInstance();
                            }
                            catch (ClassNotFoundException | InstantiationException | IllegalAccessException ex) {
                                LocaleServiceProviderPool.config(SPILocaleProviderAdapter.class, ex.toString());
                                return null;
                            }
                        }
                        ((SPILocaleProviderAdapter.Delegate)localeServiceProvider).addImpl(localeServiceProvider2);
                    }
                    return localeServiceProvider;
                }
            });
        }
        catch (PrivilegedActionException ex) {
            LocaleServiceProviderPool.config(SPILocaleProviderAdapter.class, ex.toString());
            return null;
        }
    }
    
    private static <P extends LocaleServiceProvider> P getImpl(Map<Locale, P> map, Locale locale) {
        Iterator<Locale> iterator = LocaleServiceProviderPool.getLookupLocales(locale).iterator();
        while (iterator.hasNext()) {
            LocaleServiceProvider localeServiceProvider = (P)map.get((Object)iterator.next());
            if (localeServiceProvider != null) {
                return localeServiceProvider;
            }
            continue;
        }
        return null;
    }
    
    static LocaleServiceProvider access$000(Map map, Locale locale) {
        return getImpl(map, locale);
    }
    
    static class BreakIteratorProviderDelegate extends BreakIteratorProvider implements SPILocaleProviderAdapter.Delegate<BreakIteratorProvider>
    {
        private ConcurrentMap<Locale, BreakIteratorProvider> map;
        
        BreakIteratorProviderDelegate() {
            this.map = new ConcurrentHashMap();
        }
        
        @Override
        public void addImpl(BreakIteratorProvider breakIteratorProvider) {
            Locale[] availableLocales = breakIteratorProvider.getAvailableLocales();
            for (int length = availableLocales.length, i = 0; i < length; ++i) {
                this.map.putIfAbsent(availableLocales[i], breakIteratorProvider);
            }
        }
        
        @Override
        public BreakIteratorProvider getImpl(Locale locale) {
            return (BreakIteratorProvider)SPILocaleProviderAdapter.access$000(this.map, locale);
        }
        
        @Override
        public Locale[] getAvailableLocales() {
            return (Locale[])this.map.keySet().toArray(new Locale[0]);
        }
        
        @Override
        public boolean isSupportedLocale(Locale locale) {
            return this.map.containsKey(locale);
        }
        
        @Override
        public BreakIterator getWordInstance(Locale locale) {
            BreakIteratorProvider impl = this.getImpl(locale);
            assert impl != null;
            return impl.getWordInstance(locale);
        }
        
        @Override
        public BreakIterator getLineInstance(Locale locale) {
            BreakIteratorProvider impl = this.getImpl(locale);
            assert impl != null;
            return impl.getLineInstance(locale);
        }
        
        @Override
        public BreakIterator getCharacterInstance(Locale locale) {
            BreakIteratorProvider impl = this.getImpl(locale);
            assert impl != null;
            return impl.getCharacterInstance(locale);
        }
        
        @Override
        public BreakIterator getSentenceInstance(Locale locale) {
            BreakIteratorProvider impl = this.getImpl(locale);
            assert impl != null;
            return impl.getSentenceInstance(locale);
        }
    }
    
    static class CollatorProviderDelegate extends CollatorProvider implements SPILocaleProviderAdapter.Delegate<CollatorProvider>
    {
        private ConcurrentMap<Locale, CollatorProvider> map;
        
        CollatorProviderDelegate() {
            this.map = new ConcurrentHashMap();
        }
        
        @Override
        public void addImpl(CollatorProvider collatorProvider) {
            Locale[] availableLocales = collatorProvider.getAvailableLocales();
            for (int length = availableLocales.length, i = 0; i < length; ++i) {
                this.map.putIfAbsent(availableLocales[i], collatorProvider);
            }
        }
        
        @Override
        public CollatorProvider getImpl(Locale locale) {
            return (CollatorProvider)SPILocaleProviderAdapter.access$000(this.map, locale);
        }
        
        @Override
        public Locale[] getAvailableLocales() {
            return (Locale[])this.map.keySet().toArray(new Locale[0]);
        }
        
        @Override
        public boolean isSupportedLocale(Locale locale) {
            return this.map.containsKey(locale);
        }
        
        @Override
        public Collator getInstance(Locale locale) {
            CollatorProvider impl = this.getImpl(locale);
            assert impl != null;
            return impl.getInstance(locale);
        }
    }
    
    static class DateFormatProviderDelegate extends DateFormatProvider implements SPILocaleProviderAdapter.Delegate<DateFormatProvider>
    {
        private ConcurrentMap<Locale, DateFormatProvider> map;
        
        DateFormatProviderDelegate() {
            this.map = new ConcurrentHashMap();
        }
        
        @Override
        public void addImpl(DateFormatProvider dateFormatProvider) {
            Locale[] availableLocales = dateFormatProvider.getAvailableLocales();
            for (int length = availableLocales.length, i = 0; i < length; ++i) {
                this.map.putIfAbsent(availableLocales[i], dateFormatProvider);
            }
        }
        
        @Override
        public DateFormatProvider getImpl(Locale locale) {
            return (DateFormatProvider)SPILocaleProviderAdapter.access$000(this.map, locale);
        }
        
        @Override
        public Locale[] getAvailableLocales() {
            return (Locale[])this.map.keySet().toArray(new Locale[0]);
        }
        
        @Override
        public boolean isSupportedLocale(Locale locale) {
            return this.map.containsKey(locale);
        }
        
        @Override
        public DateFormat getTimeInstance(int n, Locale locale) {
            DateFormatProvider impl = this.getImpl(locale);
            assert impl != null;
            return impl.getTimeInstance(n, locale);
        }
        
        @Override
        public DateFormat getDateInstance(int n, Locale locale) {
            DateFormatProvider impl = this.getImpl(locale);
            assert impl != null;
            return impl.getDateInstance(n, locale);
        }
        
        @Override
        public DateFormat getDateTimeInstance(int n, int n2, Locale locale) {
            DateFormatProvider impl = this.getImpl(locale);
            assert impl != null;
            return impl.getDateTimeInstance(n, n2, locale);
        }
    }
    
    static class DateFormatSymbolsProviderDelegate extends DateFormatSymbolsProvider implements SPILocaleProviderAdapter.Delegate<DateFormatSymbolsProvider>
    {
        private ConcurrentMap<Locale, DateFormatSymbolsProvider> map;
        
        DateFormatSymbolsProviderDelegate() {
            this.map = new ConcurrentHashMap();
        }
        
        @Override
        public void addImpl(DateFormatSymbolsProvider dateFormatSymbolsProvider) {
            Locale[] availableLocales = dateFormatSymbolsProvider.getAvailableLocales();
            for (int length = availableLocales.length, i = 0; i < length; ++i) {
                this.map.putIfAbsent(availableLocales[i], dateFormatSymbolsProvider);
            }
        }
        
        @Override
        public DateFormatSymbolsProvider getImpl(Locale locale) {
            return (DateFormatSymbolsProvider)SPILocaleProviderAdapter.access$000(this.map, locale);
        }
        
        @Override
        public Locale[] getAvailableLocales() {
            return (Locale[])this.map.keySet().toArray(new Locale[0]);
        }
        
        @Override
        public boolean isSupportedLocale(Locale locale) {
            return this.map.containsKey(locale);
        }
        
        @Override
        public DateFormatSymbols getInstance(Locale locale) {
            DateFormatSymbolsProvider impl = this.getImpl(locale);
            assert impl != null;
            return impl.getInstance(locale);
        }
    }
    
    static class DecimalFormatSymbolsProviderDelegate extends DecimalFormatSymbolsProvider implements SPILocaleProviderAdapter.Delegate<DecimalFormatSymbolsProvider>
    {
        private ConcurrentMap<Locale, DecimalFormatSymbolsProvider> map;
        
        DecimalFormatSymbolsProviderDelegate() {
            this.map = new ConcurrentHashMap();
        }
        
        @Override
        public void addImpl(DecimalFormatSymbolsProvider decimalFormatSymbolsProvider) {
            Locale[] availableLocales = decimalFormatSymbolsProvider.getAvailableLocales();
            for (int length = availableLocales.length, i = 0; i < length; ++i) {
                this.map.putIfAbsent(availableLocales[i], decimalFormatSymbolsProvider);
            }
        }
        
        @Override
        public DecimalFormatSymbolsProvider getImpl(Locale locale) {
            return (DecimalFormatSymbolsProvider)SPILocaleProviderAdapter.access$000(this.map, locale);
        }
        
        @Override
        public Locale[] getAvailableLocales() {
            return (Locale[])this.map.keySet().toArray(new Locale[0]);
        }
        
        @Override
        public boolean isSupportedLocale(Locale locale) {
            return this.map.containsKey(locale);
        }
        
        @Override
        public DecimalFormatSymbols getInstance(Locale locale) {
            DecimalFormatSymbolsProvider impl = this.getImpl(locale);
            assert impl != null;
            return impl.getInstance(locale);
        }
    }
    
    static class NumberFormatProviderDelegate extends NumberFormatProvider implements SPILocaleProviderAdapter.Delegate<NumberFormatProvider>
    {
        private ConcurrentMap<Locale, NumberFormatProvider> map;
        
        NumberFormatProviderDelegate() {
            this.map = new ConcurrentHashMap();
        }
        
        @Override
        public void addImpl(NumberFormatProvider numberFormatProvider) {
            Locale[] availableLocales = numberFormatProvider.getAvailableLocales();
            for (int length = availableLocales.length, i = 0; i < length; ++i) {
                this.map.putIfAbsent(availableLocales[i], numberFormatProvider);
            }
        }
        
        @Override
        public NumberFormatProvider getImpl(Locale locale) {
            return (NumberFormatProvider)SPILocaleProviderAdapter.access$000(this.map, locale);
        }
        
        @Override
        public Locale[] getAvailableLocales() {
            return (Locale[])this.map.keySet().toArray(new Locale[0]);
        }
        
        @Override
        public boolean isSupportedLocale(Locale locale) {
            return this.map.containsKey(locale);
        }
        
        @Override
        public NumberFormat getCurrencyInstance(Locale locale) {
            NumberFormatProvider impl = this.getImpl(locale);
            assert impl != null;
            return impl.getCurrencyInstance(locale);
        }
        
        @Override
        public NumberFormat getIntegerInstance(Locale locale) {
            NumberFormatProvider impl = this.getImpl(locale);
            assert impl != null;
            return impl.getIntegerInstance(locale);
        }
        
        @Override
        public NumberFormat getNumberInstance(Locale locale) {
            NumberFormatProvider impl = this.getImpl(locale);
            assert impl != null;
            return impl.getNumberInstance(locale);
        }
        
        @Override
        public NumberFormat getPercentInstance(Locale locale) {
            NumberFormatProvider impl = this.getImpl(locale);
            assert impl != null;
            return impl.getPercentInstance(locale);
        }
    }
    
    static class CalendarDataProviderDelegate extends CalendarDataProvider implements SPILocaleProviderAdapter.Delegate<CalendarDataProvider>
    {
        private ConcurrentMap<Locale, CalendarDataProvider> map;
        
        CalendarDataProviderDelegate() {
            this.map = new ConcurrentHashMap();
        }
        
        @Override
        public void addImpl(CalendarDataProvider calendarDataProvider) {
            Locale[] availableLocales = calendarDataProvider.getAvailableLocales();
            for (int length = availableLocales.length, i = 0; i < length; ++i) {
                this.map.putIfAbsent(availableLocales[i], calendarDataProvider);
            }
        }
        
        @Override
        public CalendarDataProvider getImpl(Locale locale) {
            return (CalendarDataProvider)SPILocaleProviderAdapter.access$000(this.map, locale);
        }
        
        @Override
        public Locale[] getAvailableLocales() {
            return (Locale[])this.map.keySet().toArray(new Locale[0]);
        }
        
        @Override
        public boolean isSupportedLocale(Locale locale) {
            return this.map.containsKey(locale);
        }
        
        @Override
        public int getFirstDayOfWeek(Locale locale) {
            CalendarDataProvider impl = this.getImpl(locale);
            assert impl != null;
            return impl.getFirstDayOfWeek(locale);
        }
        
        @Override
        public int getMinimalDaysInFirstWeek(Locale locale) {
            CalendarDataProvider impl = this.getImpl(locale);
            assert impl != null;
            return impl.getMinimalDaysInFirstWeek(locale);
        }
    }
    
    static class CalendarNameProviderDelegate extends CalendarNameProvider implements SPILocaleProviderAdapter.Delegate<CalendarNameProvider>
    {
        private ConcurrentMap<Locale, CalendarNameProvider> map;
        
        CalendarNameProviderDelegate() {
            this.map = new ConcurrentHashMap();
        }
        
        @Override
        public void addImpl(CalendarNameProvider calendarNameProvider) {
            Locale[] availableLocales = calendarNameProvider.getAvailableLocales();
            for (int length = availableLocales.length, i = 0; i < length; ++i) {
                this.map.putIfAbsent(availableLocales[i], calendarNameProvider);
            }
        }
        
        @Override
        public CalendarNameProvider getImpl(Locale locale) {
            return (CalendarNameProvider)SPILocaleProviderAdapter.access$000(this.map, locale);
        }
        
        @Override
        public Locale[] getAvailableLocales() {
            return (Locale[])this.map.keySet().toArray(new Locale[0]);
        }
        
        @Override
        public boolean isSupportedLocale(Locale locale) {
            return this.map.containsKey(locale);
        }
        
        @Override
        public String getDisplayName(String s, int n, int n2, int n3, Locale locale) {
            CalendarNameProvider impl = this.getImpl(locale);
            assert impl != null;
            return impl.getDisplayName(s, n, n2, n3, locale);
        }
        
        @Override
        public Map<String, Integer> getDisplayNames(String s, int n, int n2, Locale locale) {
            CalendarNameProvider impl = this.getImpl(locale);
            assert impl != null;
            return impl.getDisplayNames(s, n, n2, locale);
        }
    }
    
    static class CurrencyNameProviderDelegate extends CurrencyNameProvider implements SPILocaleProviderAdapter.Delegate<CurrencyNameProvider>
    {
        private ConcurrentMap<Locale, CurrencyNameProvider> map;
        
        CurrencyNameProviderDelegate() {
            this.map = new ConcurrentHashMap();
        }
        
        @Override
        public void addImpl(CurrencyNameProvider currencyNameProvider) {
            Locale[] availableLocales = currencyNameProvider.getAvailableLocales();
            for (int length = availableLocales.length, i = 0; i < length; ++i) {
                this.map.putIfAbsent(availableLocales[i], currencyNameProvider);
            }
        }
        
        @Override
        public CurrencyNameProvider getImpl(Locale locale) {
            return (CurrencyNameProvider)SPILocaleProviderAdapter.access$000(this.map, locale);
        }
        
        @Override
        public Locale[] getAvailableLocales() {
            return (Locale[])this.map.keySet().toArray(new Locale[0]);
        }
        
        @Override
        public boolean isSupportedLocale(Locale locale) {
            return this.map.containsKey(locale);
        }
        
        @Override
        public String getSymbol(String s, Locale locale) {
            CurrencyNameProvider impl = this.getImpl(locale);
            assert impl != null;
            return impl.getSymbol(s, locale);
        }
        
        @Override
        public String getDisplayName(String currencyCode, Locale locale) {
            CurrencyNameProvider impl = this.getImpl(locale);
            assert impl != null;
            return impl.getDisplayName(currencyCode, locale);
        }
    }
    
    static class LocaleNameProviderDelegate extends LocaleNameProvider implements SPILocaleProviderAdapter.Delegate<LocaleNameProvider>
    {
        private ConcurrentMap<Locale, LocaleNameProvider> map;
        
        LocaleNameProviderDelegate() {
            this.map = new ConcurrentHashMap();
        }
        
        @Override
        public void addImpl(LocaleNameProvider localeNameProvider) {
            Locale[] availableLocales = localeNameProvider.getAvailableLocales();
            for (int length = availableLocales.length, i = 0; i < length; ++i) {
                this.map.putIfAbsent(availableLocales[i], localeNameProvider);
            }
        }
        
        @Override
        public LocaleNameProvider getImpl(Locale locale) {
            return (LocaleNameProvider)SPILocaleProviderAdapter.access$000(this.map, locale);
        }
        
        @Override
        public Locale[] getAvailableLocales() {
            return (Locale[])this.map.keySet().toArray(new Locale[0]);
        }
        
        @Override
        public boolean isSupportedLocale(Locale locale) {
            return this.map.containsKey(locale);
        }
        
        @Override
        public String getDisplayLanguage(String s, Locale locale) {
            LocaleNameProvider impl = this.getImpl(locale);
            assert impl != null;
            return impl.getDisplayLanguage(s, locale);
        }
        
        @Override
        public String getDisplayScript(String scriptCode, Locale locale) {
            LocaleNameProvider impl = this.getImpl(locale);
            assert impl != null;
            return impl.getDisplayScript(scriptCode, locale);
        }
        
        @Override
        public String getDisplayCountry(String s, Locale locale) {
            LocaleNameProvider impl = this.getImpl(locale);
            assert impl != null;
            return impl.getDisplayCountry(s, locale);
        }
        
        @Override
        public String getDisplayVariant(String s, Locale locale) {
            LocaleNameProvider impl = this.getImpl(locale);
            assert impl != null;
            return impl.getDisplayVariant(s, locale);
        }
    }
    
    static class TimeZoneNameProviderDelegate extends TimeZoneNameProvider implements SPILocaleProviderAdapter.Delegate<TimeZoneNameProvider>
    {
        private ConcurrentMap<Locale, TimeZoneNameProvider> map;
        
        TimeZoneNameProviderDelegate() {
            this.map = new ConcurrentHashMap();
        }
        
        @Override
        public void addImpl(TimeZoneNameProvider timeZoneNameProvider) {
            Locale[] availableLocales = timeZoneNameProvider.getAvailableLocales();
            for (int length = availableLocales.length, i = 0; i < length; ++i) {
                this.map.putIfAbsent(availableLocales[i], timeZoneNameProvider);
            }
        }
        
        @Override
        public TimeZoneNameProvider getImpl(Locale locale) {
            return (TimeZoneNameProvider)SPILocaleProviderAdapter.access$000(this.map, locale);
        }
        
        @Override
        public Locale[] getAvailableLocales() {
            return (Locale[])this.map.keySet().toArray(new Locale[0]);
        }
        
        @Override
        public boolean isSupportedLocale(Locale locale) {
            return this.map.containsKey(locale);
        }
        
        @Override
        public String getDisplayName(String s, boolean b, int n, Locale locale) {
            TimeZoneNameProvider impl = this.getImpl(locale);
            assert impl != null;
            return impl.getDisplayName(s, b, n, locale);
        }
        
        @Override
        public String getGenericDisplayName(String id, int style, Locale locale) {
            TimeZoneNameProvider impl = this.getImpl(locale);
            assert impl != null;
            return impl.getGenericDisplayName(id, style, locale);
        }
    }
    
    interface Delegate<P extends LocaleServiceProvider>
    {
        public abstract void addImpl(P p0);
        
        public abstract P getImpl(Locale p0);
    }
}
